<?php
/**
 * Copyright © 2013-2017 Magento, Inc. All rights reserved.
 * See COPYING.txt for license details.
 */
namespace FiloBlu\DiscountRule\Model;


use Magento\Framework\Model\AbstractModel;
use Magento\Quote\Api\Data\TotalsInterface;
use Magento\SalesRule\Model\Rule\Condition\Address as SalesRuleAddress;

class Address extends \Magento\SalesRule\Model\Rule\Condition\Address
{
    const KEY_SUBTOTAL_WITH_DISCOUNT_NO_SHIPPING = 'subtotal_with_discount_exclude_shipping';
    /**
     * Load attribute options
     *
     * @return $this
     */
    public function loadAttributeOptions()
    {
        $attributes = [
            'base_subtotal' => __('Subtotal'),
            'total_qty' => __('Total Items Quantity'),
            'weight' => __('Total Weight'),
            'shipping_method' => __('Shipping Method'),
            'postcode' => __('Shipping Postcode'),
            'region' => __('Shipping Region'),
            'region_id' => __('Shipping State/Province'),
            'country_id' => __('Shipping Country'),
            'base_subtotal_incl_tax' => __('Subtotal (Including Tax)'),
            'base_grand_total' => __('Base Grand Total'),
            'subtotal_with_discount' =>__('Subtotal With Discount'),
            'base_subtotal_with_discount' =>__('Base Subtotal With Discount'),
            'grand_total' => __('Grand Total'),
            'subtotal_with_discount_exclude_shipping' => __('Subtotal with discounts no shipping (Incl. Tax)')
        ];

        $this->setAttributeOption($attributes);

        return $this;
    }


    /**
     * Get input type
     *
     * @return string
     */
    public function getInputType()
    {
        switch ($this->getAttribute()) {
            case 'base_subtotal_incl_tax':
            case 'subtotal_with_discount':
            case 'base_subtotal_with_discount':
            case 'subtotal_with_discount_exclude_shipping':
            case 'base_subtotal':
            case 'grand_total':
            case 'base_grand_total':
            case 'weight':
            case 'total_qty':
                return 'numeric';

            case 'shipping_method':
            case 'payment_method':
            case 'country_id':
            case 'region_id':
                return 'select';
        }
        return 'string';


    }


    /**
     * Get value select options
     *
     * @return array|mixed
     */
    public function getValueSelectOptions()
    {
        if (!$this->hasData('value_select_options')) {
            switch ($this->getAttribute()) {
                case 'country_id':
                    $options = $this->_directoryCountry->toOptionArray();
                    break;

                case 'region_id':
                    $options = $this->_directoryAllregion->toOptionArray();
                    break;

                case 'shipping_method':
                    $options = $this->_shippingAllmethods->toOptionArray();
                    break;

                case 'payment_method':
                    $options = $this->_paymentAllmethods->toOptionArray();
                    break;

                default:
                    $options = [];
            }
            $this->setData('value_select_options', $options);
        }

    }

    /**
     * Validate Address Rule Condition
     *
     * @param \Magento\Framework\Model\AbstractModel $model
     * @return bool
     */
    public function validate(\Magento\Framework\Model\AbstractModel $model)
    {
        $address = $model;
        if (!$address instanceof \Magento\Quote\Model\Quote\Address) {
            if ($model->getQuote()->isVirtual()) {
                $address = $model->getQuote()->getBillingAddress();
            } else {
                $address = $model->getQuote()->getShippingAddress();
            }
        }

        if ('payment_method' == $this->getAttribute() && !$address->hasPaymentMethod()) {
            $address->setPaymentMethod($model->getQuote()->getPayment()->getMethod());

        }

        $address->setData(self::KEY_SUBTOTAL_WITH_DISCOUNT_NO_SHIPPING, $address->getSubtotalInclTax() + $address->getDiscountAmount());
       // $address->setBaseSubtotalWithDiscount($address->getBaseSubtotal() + $address->getDiscountAmount());

        return parent::validate($address);
    }
}
